/*
 * ===========================================================================================
 *
 *       Filename:  clic.c
 *
 *    Description:  clic csp driver.
 *
 *        Version:  Melis3.0 
 *         Create:  2021-01-26 14:23:46
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2021-01-28 09:23:52
 *
 * ===========================================================================================
 */

#include <typedef.h>
#include <stdlib.h>
#include <excep.h>
#include <debug.h>
#include <arch.h>
#include <csr.h>
#include <log.h>
#include <stdio.h>
#include <io.h>

#define CSR_PLIC_BASE            0xfc1
#define C910_PLIC_CLINT_OFFSET   0x04000000
static uint64_t mmode_peripheral_addr_base;
static uint64_t clic_int_control_addr_base;
static uint64_t clic_int_control_addr_comp;
static uint64_t clic_int_control_addr_mtime;

int clic_driver_init(void)
{
    mmode_peripheral_addr_base = csr_read(CSR_PLIC_BASE);
    if(mmode_peripheral_addr_base == 0)
    {
        printf("%s line %d, fatal error, peripheral address is null!\n", __func__, __LINE__);
        return -1;
    }

    clic_int_control_addr_base = mmode_peripheral_addr_base + C910_PLIC_CLINT_OFFSET;

    clic_int_control_addr_mtime = clic_int_control_addr_base + 0xbff8;
    clic_int_control_addr_comp = clic_int_control_addr_base + 0x4000;

    printf("%s line %d, plic base 0x%lx, clic base 0x%lx\n", \
        __func__, __LINE__, mmode_peripheral_addr_base, clic_int_control_addr_base);

    return 0;
}

uint64_t sunxi_clic_read_mtime(void)
{
    uint32_t lo, hi;
    unsigned long addr = clic_int_control_addr_mtime;

    do{
        hi = readl_relaxed((uint32_t *)addr + 1);
        lo = readl_relaxed((uint32_t *)addr);
    } while (hi != readl_relaxed((uint32_t *)addr + 1));

    return ((uint64_t)hi << 32) | (uint64_t)lo;
}

void sunxi_program_timer_next_event(unsigned long next_compare)
{
    unsigned int mask = -1U;
    unsigned long value = next_compare;
    unsigned long addr = clic_int_control_addr_comp;
    
    writel_relaxed(value & mask, (void *)(addr));
    writel_relaxed(value >> 32, (void *)(addr) + 0x04);
}
