/*
 * ===========================================================================================
 *
 *       Filename:  csp_uart.c
 *
 *    Description:  uart csp impl.
 *
 *        Version:  Melis3.0
 *         Create:  2020-06-23 18:37:54
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2021-01-26 10:45:42
 *
 * ===========================================================================================
 */

#include <../sbi/sbi.h>

#define SUNXI_UART_ADDR      0x02500000
#define SUNXI_UART_THR       0x0
#define SUNXI_UART_RBR       0x0
#define SUNXI_UART_USR       0x1f
#define SUNXI_UART_USR_NF    0x02
#define SUNXI_UART_USR_RFNE  0x04

static volatile unsigned int *sunxi_console_regbase;

static int sunxi_uart_init(unsigned long base)
{
    sunxi_console_regbase = (unsigned int*)base;
    return 0;
}

int sunxi_console_early_init(void)
{
    return sunxi_uart_init(SUNXI_UART_ADDR);
}

void sunxi_uart_putc(char ch)
{
    while ((sunxi_console_regbase[SUNXI_UART_USR] & SUNXI_UART_USR_NF) == 0);

    sunxi_console_regbase[SUNXI_UART_THR] = ch;
}

int sunxi_uart_getc(void)
{
    if ((sunxi_console_regbase[SUNXI_UART_USR] & SUNXI_UART_USR_RFNE) != 0)
    {
        return sunxi_console_regbase[SUNXI_UART_RBR];
    }
    else
    {
        return -1;
    }
}

int uart_put_buffer(const char *buf, int count)
{
    int i;

    for (i = 0; i < count; i++)
    {
        sunxi_uart_putc(buf[i]);
        if(buf[i] == '\n')
        {
            sunxi_uart_putc('\r');
        }
    }

    return i;
}

int uart_get_buffer(char *buf, int count)
{
    int i, c;

    void delay_10insn(void);
    for (i = 0; i < count; i++)
    {
retry:
        c = sunxi_uart_getc();
        if (c < 0)
        {
            delay_10insn();
            delay_10insn();
            delay_10insn();
            goto retry;
        }

        buf[i] = c;
    }

    return i;
}
