/*
 * ===========================================================================================
 *
 *       Filename:  startup_gcc.S
 *
 *    Description:  bringup for f133.
 *
 *        Version:  Melis3.0 
 *         Create:  2020-06-24 08:58:38
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2021-01-25 19:11:04
 *
 * ===========================================================================================
 */
#include <csr.h>

.section ".start", "ax", @progbits
.global	_start
.option norvc
_start:
    # disable interrupt ability and mask all the interrupts.
    csrw  mie, zero
    csrw  mip, zero

    # x0 hardcoded to zero.
    li    x1,  0
    li    x2,  0
    li    x3,  0
    li    x4,  0
    li    x5,  0
    li    x6,  0
    li    x7,  0
    li    x8,  0
    li    x9,  0
    li    x10, 0
    li    x11, 0
    li    x12, 0
    li    x13, 0
    li    x14, 0
    li    x15, 0
    li    x16, 0
    li    x17, 0
    li    x18, 0
    li    x19, 0
    li    x20, 0
    li    x21, 0
    li    x22, 0
    li    x23, 0
    li    x24, 0
    li    x25, 0
    li    x26, 0
    li    x27, 0
    li    x28, 0
    li    x29, 0
    li    x30, 0
    li    x31, 0

    # Kind of the chicken/egg first problem
    # the initalization of gp needs to be protected from relaxation otherwise 
    # the initialization of gp itself will try to use a garbage 
    # gp with disastrous results.
    # without this norelax option, la gp, __global_pointer$
    # will be emitted with something like  "addi gp, gp, 0".
.option push # push the following setting of all .options to a stack. so that subsequent .option pop will restore their value.
.option norelax
    la    gp, __global_pointer$
.option pop  # pop the optios stack, restore all .options to the setting of the last .option push time.

    # auipc gp, %pcrel_hi(__global_pointer$)
    # addi  gp, gp, %pcrel_lo(1b)

    /* c910 will invalid all i-cache and d-cache automatically when reset, but this is c906 */
    /* invalid all MMU TLB Entry.*/
    sfence.vma  x0, x0

    # Following are map 0G-4G with VA==PA MODE, basically,RISCV is very very like
    # MIPS does, just difference with regiser setting manner(Like EntryHi, EntryLo and TLBWR
    # instructions in MIPS).
    # Map VA(0G-1G) to PA(0G-1G)
    li	 a0, (0x4 << 16)       # PageSize is 1G.
    li	 a1, 0x0 << (26 + 18)  # set VPN to 0 start.
    or	 a0, a0,a1             # construct smeh image.
    csrw smeh, a0              # write to smeh csr register.

    li	 a0, 0xff              # set SMEL, 0x0 maps 0x0, So = 0, C = 0, B = 0, SH = 0, D = 1, A = 1, G = 1, U = 1, XWR = 7, V = 1
    li	 a1, 0x0 << (26 + 10)  # construct PPN number with PA 0 page start.
    or	 a0, a0,a1             # or with page attribute.
    csrw smel, a0              # write to smel csr register.

    li	 a0, 0x10000000        # set TLBWR bit of smcir, random write.
    csrw smcir, a0

    # Map VA(1G-2G) to PA(1G-2G)
    li	 a0, (0x4 << 16)
    li	 a1, 0x1 << (26 + 18)
    or	 a0, a0,a1
    csrw smeh, a0

    li	 a0, 0xff
    li	 a1, 0x1 << (26 + 10)
    or	 a0, a0,a1
    csrw smel, a0

    li 	 a0, 0x10000000
    csrw smcir, a0

    # Map VA(2G-3G) to PA(2G-3G)
    li	 a0, (0x4 << 16)
    li	 a1, 0x2 << (26 + 18)
    or	 a0, a0,a1
    csrw smeh, a0

    li	 a0, 0xff
    li	 a1, 0x2 << (26 + 10)
    or	 a0, a0,a1
    csrw smel, a0

    li 	 a0, 0x10000000
    csrw smcir, a0

    # Map VA(3G-4G) to PA(3G-4G)
    li	 a0, (0x4 << 16)
    li	 a1, 0x3 << (26 + 18)
    or	 a0, a0,a1
    csrw smeh, a0

    li	 a0, 0xff
    li	 a1, 0x3 << (26 + 10)
    or	 a0, a0,a1
    csrw smel, a0

    li 	 a0, 0x10000000
    csrw smcir, a0

    # After Map, set SATP mode sv39, enable mmu.
    #li	 a0, 8 << 60
    li   a0, 0 << 60
    csrw satp, a0

    sfence.vma

    #mtevc[1~0]: 2'b00 interrupt and exception use the same unify entry point.
    #mtevc[1~0]: 2'b01 exception use the unify entry point of BASE, but interrupt use BASE + 4*IRQCODE.
    la   t0, handle_exception
    ori  t0, t0, 0x0
    csrw mtvec, t0

1:
    /* make sure that mtvec is updated */
    csrr t1, mtvec
    bne  t1, t0, 1b

    
    /* Clear and Set FS status, we are using FPU In kernel. */
    # clear SD temporaly.disable and interrupt.
    li    t0, SR_FS_DIRTY
    csrc  mstatus, t0

    # initial status to set the fpu registers.
    li    t0, SR_FS_INITIAL
    csrs  mstatus, t0

    # follwing code would change state of fpu registers
    # would set sr_sd again.

    # clear the floating-point swap control and status register.
    # abandon the original value.
    fscsr   x0, x0

    # clear the GPRS of FPU.
    fmv.d.x  f0,  x0
    fmv.d.x  f1,  x0
    fmv.d.x  f2,  x0
    fmv.d.x  f3,  x0
    fmv.d.x  f4,  x0
    fmv.d.x  f5,  x0
    fmv.d.x  f6,  x0
    fmv.d.x  f7,  x0
    fmv.d.x  f8,  x0
    fmv.d.x  f9,  x0
    fmv.d.x  f10, x0
    fmv.d.x  f11, x0
    fmv.d.x  f12, x0
    fmv.d.x  f13, x0
    fmv.d.x  f14, x0
    fmv.d.x  f15, x0
    fmv.d.x  f16, x0
    fmv.d.x  f17, x0
    fmv.d.x  f18, x0
    fmv.d.x  f19, x0
    fmv.d.x  f20, x0
    fmv.d.x  f21, x0
    fmv.d.x  f22, x0
    fmv.d.x  f23, x0
    fmv.d.x  f24, x0
    fmv.d.x  f25, x0
    fmv.d.x  f26, x0
    fmv.d.x  f27, x0
    fmv.d.x  f28, x0
    fmv.d.x  f29, x0
    fmv.d.x  f30, x0
    fmv.d.x  f31, x0

    # clear mscratch/sscratch register.
    csrw   mscratch,zero
    csrw   sscratch,zero
    # clear SD temporaly. and disable interrupt.
    li    t0, SR_FS_DIRTY
    csrc  mstatus, t0

    # re-initial status to initial status.
    li    t0, SR_FS_INITIAL
    csrs  mstatus, t0

    la   t0, __bss_start
    la   t1, __bss_end
    ble  t1, t0, clear_bss_done

clear_bss:
    sd   zero, (t0)
    add  t0, t0, 8
    blt  t0, t1, clear_bss

clear_bss_done:

    la    t0, __init_process_stack_start__
    la    t1, __init_process_stack_end__
    # fill stack with 'ebreak' instructions.
    li    t2, 0x9002900290029002

1:
    sd    t2, 0(t0)
    add   t0, t0, 8
    blt   t0, t1, 1b

    # prepare stack for c.
    la    sp, __init_process_stack_end__

    # prepare jump to c code.
.option rvc
to_c:
    # Para:
    # a0: boot cost 0 nanoseconds just for dummy
    # a1: report misa to system software.
    # a2: sbi start.
    # a3: sbi end.
    li   a0,  0
    csrr a1,  misa
    la   a2, __sbi_binary_start
    la   a3, __sbi_binary_end

    sfence.vma

    la    t0, start_kernel
    jalr  zero, t0
    nop
    nop

.align 3

_load_start:
    .dword 0x0

_link_start:
    .dword _firmware_start

_link_end:
    .dword _firmware_end

.end
