/*
 * ===========================================================================================
 *
 *       Filename:  vmem_test.c
 *
 *    Description:  virtual memory test.
 *
 *        Version:  Melis3.0
 *         Create:  2020-07-08 13:58:17
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2020-07-28 15:56:41
 *
 * ===========================================================================================
 */

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include "page.h"
#include "pgtable.h"
#include <typedef.h>
#include <rtthread.h>
#include <debug.h>
#include <string.h>
#include <ktimer.h>
#include <stdio.h>
#include <sbi.h>
#include <log.h>

int module_alloc(void *target_vmaddr, unsigned long map_size, bool executable);
int module_free(void *target_vmaddr, unsigned long free_size);
unsigned int pic_code[] =
{
    0xfe010113,              // addi    sp,sp,-32
    0x00813c23,              // sd      s0,24(sp)
    0x02010413,              // addi    s0,sp,32
    0xfea43423,              // sd      a0,-24(s0)
    0xfeb43023,              // sd      a1,-32(s0)
    0xfe843703,              // ld      a4,-24(s0)
    0xfe043783,              // ld      a5,-32(s0)
    0x00f707b3,              // add     a5,a4,a5
    0x00078513,              // mv      a0,a5
    0x01813403,              // ld      s0,24(sp)
    0x02010113,              // addi    sp,sp,32
    0x00008067,              // ret
};

static void memory_info(void)
{
    rt_uint32_t total, used, max_used;

    rt_memory_info(&total, &used, &max_used);

    printf("\n\rmemory info:\n\r");
    printf("    Total  0x%08x\n\r" \
           "    Used   0x%08x\r\n" \
           "    Max    0x%08x\r\n" \
           "    Margin 0x%08x\n\r", \
           total, used, max_used, total - max_used);
}

void vmem_create_test(void)
{
    unsigned long *ptr = (unsigned long *)0xe0000000;

    //store op. trigger store page fault before map.
    /*
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     *[DBG]: [riscv_cpu_handle_exception:0145]:                   error STORE_PAGE_FAULT
     */
#if 0
    *ptr = 0x5a5a5a5a5a5a5a5a;
#endif

    unsigned long *ptr_write;
    unsigned long *ptr_read;
    struct timespec64 ts64;
    while (1)
    {
        // create virtual memory start from 0xe000_0000, size 1M.
        int ret = module_alloc((void *)0xe0000000, 0x200000, true);
        if (ret != 0)
        {
            printf("fatal error, create vmem failure.\n");
            software_break();
            return;
        }
        printf("create vmem ok\n");

        // now fine, the target address would be 0x5a5a5a5a5a5a5a.
        /*
         *(gdb) x /4gx 0xe0000000
         *0xe0000000:     0x5a5a5a5a5a5a5a5a      0x0000000000000000
         *0xe0000010:     0x0000000000000000      0x0000000000000000
         *(gdb)
         */

        for (ptr_write = ptr ; ptr_write < (unsigned long *)0xe0200000; ptr_write ++)
        {
            *ptr_write = 0x5a5a5a5a5a5a5a5a;
        }

        for (ptr_read = ptr ; ptr_read < (unsigned long *)0xe0200000; ptr_read ++)
        {
            if (*ptr_read != 0x5a5a5a5a5a5a5a5a)
            {
                printf("fatal error, ptr_read 0x%p.\n", ptr_read);
                software_break();
            }
        }

        memcpy((void *)0xe0100000, pic_code, sizeof(pic_code));
        unsigned long (*entry)(unsigned long, unsigned long) = (unsigned long (*)(unsigned long, unsigned long))0xe0100000;
        unsigned long retval = entry(33, 66);
        printf("vmem create test pass! return val %ld\n", retval);
        module_free((void *)0xe0000000, 0x200000);
        printf("vmem free.\n");
        memory_info();
        do_gettimeofday(&ts64);
        printf("%s line %d, sec: %ld, nsec %d.\n", __func__, __LINE__, ts64.tv_sec, ts64.tv_nsec);
    }
    software_break();
}
