/*
 * ===========================================================================================
 *
 *       Filename:  init.c
 *
 *    Description:  bringup file.
 *
 *        Version:  Melis3.0
 *         Create:  2020-08-17 12:07:20
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2021-01-29 14:31:58
 *
 * ===========================================================================================
 */
#include <stdlib.h>
#include <stdio.h>
#include <log.h>
#include <init.h>
#include <sysinit.h>
#include <string.h>
#include <timex.h>
#include <debug.h>
#include <port.h>
#include <cmpxchg.h>
#include <sunxi_drv_gpio.h>
#include <FreeRTOS.h>
#include <task.h>


#define  STACK_CHECK_MAGICNUMBER    (0x5a5a5a5adeadbeef)
/* ----------------------------------------------------------------------------*/
/** @brief  <++> */
/* ----------------------------------------------------------------------------*/
uint8_t melis_kernel_running = 0;
/* ----------------------------------------------------------------------------*/
/** @brief  <++> */
/* ----------------------------------------------------------------------------*/
uint8_t *melis_ramdisk_base;
/* ----------------------------------------------------------------------------*/
/** @brief  <++> */
/* ----------------------------------------------------------------------------*/
uint32_t melis_ramdisk_size;

unsigned long __stack_chk_guard;

#if defined CONFIG_BOOT_LOGO_BMP
uint32_t melis_logo_bmp_size = 0;
uint8_t *melis_logo_bmp_base;
#endif

extern int          drv_clock_init(void);
extern int          multiple_console_init(void);
extern void         do_initcalls(void);
extern void         plic_init(void);
extern void         system_tick_init(void);
extern int          interrupt_init(void);
extern int          finsh_system_init(void);
extern void         decode_misa(unsigned long);
extern void         timekeeping_init(void);
extern void         trap_init(void);
extern int32_t      ksrv_init(void);
extern int32_t      input_init(void);
extern uint32_t     fsys_init(void);
extern int32_t      dmsdev_init(void);
extern int32_t      ramdiskdev_init(void);
extern int          dfs_init(void);

static noinline void copy_ramdisk(void)
{
#if 0
    extern unsigned char blob_input_ramdisk_start[];
    extern unsigned char blob_input_ramdisk_end[];

    melis_ramdisk_size = blob_input_ramdisk_end - blob_input_ramdisk_start;
    melis_ramdisk_base = blob_input_ramdisk_start;

    __log("ramdisk: base %p, size %d.", melis_ramdisk_base, melis_ramdisk_size);
#endif
    return;
}

static void init_bss(void)
{
#if 0
    unsigned long *dst;
    extern unsigned long __bss_start[];
    extern unsigned long __bss_end[];

    dst = __bss_start;
    while (dst < __bss_end)
    {
        *dst++ = 0;
    }
#endif
}

static int32_t awos_bsp_init(void)
{
#if 0
    system_tick_init();
    plic_init();
    interrupt_init();
    copy_ramdisk();
#endif
    return 0;
}

void vAssertCalled( void )
{
    printf("%s line %d.\n", __func__, __LINE__);
    while(1);
}
void vApplicationTickHook( void )
{
    //printf("%s line %d.\n", __func__, __LINE__);
}

void vApplicationMallocFailedHook( void )
{

}

void vApplicationStackOverflowHook( TaskHandle_t xTask, char * pcTaskName )
{

}

void C906_Default_IRQHandler(void)
{
    printf("%s line %d.\n", __func__, __LINE__);
    return;
}

void cpu_app_entry(void *para)
{
    while(1)
    {
        printf("%s line %d cur:%s.\n", __func__, __LINE__, pcTaskGetName(NULL));
        vTaskDelay(100);
    }
}

static void app_init(void)
{
    BaseType_t ret;

    ret = xTaskCreate(cpu_app_entry, (signed portCHAR *) "init-thread-0", 4096, NULL, 31, NULL);
    if (ret != pdPASS)
    {
        while(1)
        {
            printf("%s line %d.\n", __func__, __LINE__);
        }
    }

    ret = xTaskCreate(cpu_app_entry, (signed portCHAR *) "init-thread-1", 4096, NULL, 31, NULL);
    if (ret != pdPASS)
    {
        while(1)
        {
            printf("%s line %d.\n", __func__, __LINE__);
        }
    }

    return;
}

asm(".option norvc");
int clic_driver_init(void);
int sunxi_console_early_init(void);
void hpm_get_icache_miss_rate(void);
void start_kernel(unsigned long spl_nanosec, unsigned long misa, unsigned long sbi_start, unsigned long sbi_end)
{
    //early printf from uart.
    sunxi_console_early_init();

    decode_misa(misa);
    trap_init();
    timekeeping_init();

    clic_driver_init();

    system_tick_init();

    app_init();
    hpm_get_icache_miss_rate();
    vTaskStartScheduler();

    /* never come back */
    CODE_UNREACHABLE;
}
