/*
 * ===========================================================================================
 *
 *       Filename:  startup.c
 *
 *    Description:  rtkernel os startup routine.
 *
 *        Version:  Melis3.0
 *         Create:  2020-06-24 10:32:22
 *       Revision:  none
 *       Compiler:  GCC:version 7.2.1 20170904 (release),ARM/embedded-7-branch revision 255204
 *
 *         Author:  caozilong@allwinnertech.com
 *   Organization:  BU1-PSW
 *  Last Modified:  2021-01-20 17:03:42
 *
 * ===========================================================================================
 */

#include <port.h>

#ifdef CONFIG_RTTKERNEL
#include <rtthread.h>
#endif

#include <version.h>
#include <kconfig.h>
#include <_newlib_version.h>
#include <debug.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arch.h>
#include <log.h>

static const char melis_banner [] =
{
    "|                   /'\\_/`\\       (_ )  _                       /'_  )    /'_ `\\   /' _`\\                     |\n\r"
    "|                   |     |   __   | | (_)  ___  ______  _   _ (_)_) |   ( (_) |   | ( ) |                    |\n\r"
    "|                   | (_) | /'__`\\ | | | |/',__)(______)( ) ( ) _(_ <     \\__, |   | | | |                    |\n\r"
    "|                   | | | |(  ___/ | | | |\\__, \\        | \\_/ |( )_) | _     | | _ | (_) |                    |\n\r"
    "|                   (_) (_)`\\____)(___)(_)(____/        `\\___/'`\\____)(_)    (_)(_)`\\___/'                    |\n\r"
};

#define MELIS_VERSION_STRING(major, minor, patch) \
    #major"."#minor"."#patch

static const char *melis_banner_string(void)
{
    return ("V"MELIS_VERSION_STRING(3, 9, 0));
}

void show_melis_version(void)
{
    //use newlibc printf for long strings.
    printf("\n\r");
    printf("===============================================================================================================\n\r");
    printf("%s", melis_banner);
    printf("|version : %s%s%s\n\r", melis_banner_string(), XPOSTO(110), "|");
    printf("|commitid: %.*s%s%s\n\r", 100, SDK_GIT_VERSION, XPOSTO(110), "|");
    printf("|sunxiver: %x%s%s\n\r", MELIS_VERSION_CODE, XPOSTO(110), "|");
    printf("|timever : %.*s%s%s\n\r", 100, SDK_UTS_VERSION, XPOSTO(110), "|");
    printf("|compiler: %.*s%s%s\n\r", 100, GCC_VERSION, XPOSTO(110), "|");
#ifdef CONFIG_CC_OPTIMIZE_FOR_SIZE
    printf("|optimal : %.*s%s%s\n\r", 100, "-Os -g -gdwarf-2 -gstrict-dwarf", XPOSTO(110), "|");
#else
    printf("|optimal : %.*s%s%s\n\r", 100, "-O0 -g -gdwarf-2 -gstrict-dwarf", XPOSTO(110), "|");
#endif
    printf("|linker  : %.*s%s%s\n\r", 100, LNK_VERSION, XPOSTO(110), "|");
    printf("|newlibc : %.*s%s%s\n\r", 100, _NEWLIB_VERSION, XPOSTO(110), "|");
    printf("|author  : %.*s%s%s\n\r", 100, SDK_COMPILE_BY, XPOSTO(110), "|");
    printf("===============================================================================================================\n\r");
    printf("\n\r");
}

static int rt_application_init(pfun_krnl_init_t app_init)
{
#ifdef CONFIG_RTTKERNEL
    static rt_thread_t app_init_thread;

    // highest priority to system bringup.
    app_init_thread = rt_thread_create("app_init0", app_init, RT_NULL, 0x4000, 0, 10);
    RT_ASSERT(app_init_thread != RT_NULL);

    rt_thread_startup(app_init_thread);
#endif

    return 0;
}

/*
 *FPGA:
 * take range [__bss_end, 0x44000000) as heap.
 */
#define HEAP_TOP      (0x44000000)
void awos_init_bootstrap(pfun_bsp_init_t rt_system_bsp_init, pfun_krnl_init_t app_init, uint64_t ticks)
{
#ifdef CONFIG_RTTKERNEL
    void *heap_start, *heap_end;

    extern unsigned long __bss_end[];
    heap_start = (void *)__bss_end;
    heap_end   = (void *)HEAP_TOP;

    rt_system_heap_init(heap_start, heap_end);
    rt_system_timer_init();

    setbuf(stdout, NULL);
    show_melis_version();

    rt_system_scheduler_init();
    rt_system_timer_thread_init();

    if (rt_system_bsp_init)
    {
        rt_system_bsp_init();
    }

    /* initialize idle thread */
    rt_thread_idle_init();

    rt_application_init(app_init);

    //void rt_perf_init(void);
    //rt_perf_init();
    rt_system_scheduler_start();

#ifdef CONFIG_BRINGUP_IDLE
    void idle_entry(void);
    idle_entry();
#endif
#endif

    /* never come here */
    CODE_UNREACHABLE;
}
