#include <stdlib.h>
#include <stdio.h>
#include <rthw.h>
#include <rtthread.h>
#include <log.h>
#include <init.h>
#include <sysinit.h>
#include <string.h>
#include <timex.h>
#include <debug.h>
#include <port.h>
#include <cmpxchg.h>
#include "mod_slib.h"

#define ELEMENT_VADDR  0xD0000000
#define ELEMENT_BYTES  0x4000

extern int module_alloc(void *target_vmaddr, unsigned long map_size, bool executable);
extern int module_free(void *target_vmaddr, unsigned long free_size);
rt_sem_t tslib_install_sem = NULL, tslib_uninstall_sem = NULL;
__u32 slib_mid = 0;

static void slib_modinstall_test_entry(void *para)
{
    rt_err_t ret = 0l;

    rt_kprintf("[%s %d]\r\n", __FUNCTION__, __LINE__);

    RT_ASSERT(tslib_install_sem != RT_NULL);
    RT_ASSERT(tslib_uninstall_sem != RT_NULL);

    while (1)
    {
        ret = rt_sem_take(tslib_install_sem, RT_WAITING_FOREVER);
        if (RT_EOK != ret)
        {
            rt_kprintf("========================[%s %d]: take sem err!\r\n", __FUNCTION__, __LINE__);
            rt_thread_delay(50);
            continue;
        }
        else
        {
            if (0 == slib_mid)
            {
                slib_mid = esMODS_MInstall("/mnt/E/slib.mod", 0);
            }
            if (slib_mid)
            {
                char *buffer = (char *)ELEMENT_VADDR;
                int ret = module_alloc((void *)ELEMENT_VADDR, ELEMENT_BYTES, false);
                RT_ASSERT(0 == ret);
                SLIB_memclr(buffer, ELEMENT_BYTES);
                SLIB_memset(buffer, 's', ELEMENT_BYTES);
                for (int i = 0; i < ELEMENT_BYTES; i ++)
                {
                    if(i < ELEMENT_BYTES - 1) buffer[i] = (char)SLIB_toupper((int)buffer[i]);
                    if ('S' != buffer[i])
                    {
                        rt_kprintf("========================[%s %d]: buffer[0x%X]=%c\r\n", __FUNCTION__, __LINE__, i, buffer[i]);
                    }
                }
            }

            rt_thread_delay(200);
            rt_kprintf("========================[%s %d] switch to uninstall!\r\n", __FUNCTION__, __LINE__);
            rt_sem_release(tslib_uninstall_sem);
        }
    }
}

static void slib_moduninstall_test_entry(void *para)
{
    rt_err_t ret = 0;

    rt_kprintf("[%s %d]\r\n", __FUNCTION__, __LINE__);

    RT_ASSERT(tslib_install_sem != RT_NULL);
    RT_ASSERT(tslib_uninstall_sem != RT_NULL);

    while (1)
    {
        ret = rt_sem_take(tslib_uninstall_sem, RT_WAITING_FOREVER);
        if (RT_EOK != ret)
        {
            rt_kprintf("========================[%s %d]: take sem err!\r\n", __FUNCTION__, __LINE__);
            rt_thread_delay(50);
            continue;
        }
        else
        {
            module_free((void *)ELEMENT_VADDR, ELEMENT_BYTES);
            if (slib_mid)
            {
                s32 ret = 0;
                ret = esMODS_MUninstall((__u8)slib_mid);
                RT_ASSERT(EPDK_OK == ret);
                slib_mid = 0;
            }
            rt_thread_delay(200);
            {
                rt_uint32_t total, used, max_used;

                rt_memory_info(&total, &used, &max_used);

                printf("\n\rmemory info:\n\r");
                printf("    Total  0x%08x\n\r" \
                       "    Used   0x%08x\r\n" \
                       "    Max    0x%08x\r\n" \
                       "    Margin 0x%08x\n\r", \
                       total, used, max_used, total - max_used);
            }
            rt_kprintf("========================[%s %d] switch to install!\r\n", __FUNCTION__, __LINE__);
            rt_sem_release(tslib_install_sem);
        }
    }
}


void slib_mod_test(void)
{
    static rt_thread_t slib_modinstall_thread = NULL;
    static rt_thread_t slib_moduninstall_thread = NULL;

    if (NULL == tslib_install_sem)
    {
        tslib_install_sem = rt_sem_create("tslib_sem", 1, RT_IPC_FLAG_FIFO);
        RT_ASSERT(tslib_install_sem != RT_NULL);
    }
    if (NULL == tslib_uninstall_sem)
    {
        tslib_uninstall_sem = rt_sem_create("tslib_un_sem", 0, RT_IPC_FLAG_FIFO);
        RT_ASSERT(tslib_uninstall_sem != RT_NULL);
    }

    // highest priority to system bringup.
    slib_modinstall_thread = rt_thread_create("tModIns  ", slib_modinstall_test_entry, RT_NULL, 0x4000, 1, 10);
    RT_ASSERT(slib_modinstall_thread != RT_NULL);
    rt_thread_startup(slib_modinstall_thread);

    slib_moduninstall_thread = rt_thread_create("tModUnins  ", slib_moduninstall_test_entry, RT_NULL, 0x4000, 1, 10);
    RT_ASSERT(slib_moduninstall_thread != RT_NULL);
    rt_thread_startup(slib_moduninstall_thread);
}

#ifdef CONFIG_LEGACY_TEST_RV_SYSCALL
#include <syscall.h>
void test_syscall_thread(void)
{
    while(1)
    {
        rt_thread_delay(1000);
        __log("===%s %d====\r\n",__FUNCTION__, __LINE__);
    }
}
#endif

